==> v10.5.0.tar.gz: brewman/overlord/src/app/settings/settings.component.ts <==
import { Component, OnInit } from '@angular/core';
import { FormArray, FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { BehaviorSubject } from 'rxjs';

import { environment } from '../../environments/environment';
import { AuthService } from '../auth/auth.service';
import { AccountType } from '../core/account-type';
import { ToasterService } from '../core/toaster.service';
import { ConfirmDialogComponent } from '../shared/confirm-dialog/confirm-dialog.component';

import { LockDataSource } from './lock-datasource';
import { LockInfo } from './lock-info';
import { SettingsService } from './settings.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.css'],
})
export class SettingsComponent implements OnInit {
  public lockObservable = new BehaviorSubject<LockInfo[]>([]);
  dataSource: LockDataSource = new LockDataSource(this.lockObservable);

  accountTypes: AccountType[] = [];
  voucherTypes: AccountType[] = [];

  lockInfoForm: FormGroup<{
    validFrom: FormControl<moment.Moment | null>;
    validTill: FormControl<moment.Moment | null>;
    index: FormControl<number>;
    lockOlder: FormControl<boolean>;
    olderRolling: FormControl<boolean>;
    olderDate: FormControl<moment.Moment | null>;
    olderDays: FormControl<number | null>;
    lockNewer: FormControl<boolean>;
    newerRolling: FormControl<boolean>;
    newerDate: FormControl<moment.Moment | null>;
    newerDays: FormControl<number | null>;
    accountTypes: FormArray<FormGroup<{ accountType: FormControl<boolean> }>>;
    voucherTypes: FormArray<FormGroup<{ voucherType: FormControl<boolean> }>>;
  }>;

  lockInformation: LockInfo[];
  lockOlder: boolean = false;
  olderRolling: boolean = false;
  lockNewer: boolean = false;
  newerRolling: boolean = false;

  rebaseDataForm: FormGroup<{
    date: FormControl<moment.Moment>;
  }>;

  maintenance: { enabled: boolean; user: string };

  displayedColumns = ['index', 'validity', 'voucherTypes', 'accountTypes', 'lock', 'action'];

  version: string;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    public auth: AuthService,
    private ser: SettingsService,
  ) {
    const startDate = moment().date(1);
    const finishDate = moment().date(startDate.daysInMonth());
    this.lockInfoForm = new FormGroup({
      validFrom: new FormControl<moment.Moment | null>(null),
      validTill: new FormControl<moment.Moment | null>(null),
      index: new FormControl<number>(0, { nonNullable: true }),
      lockOlder: new FormControl<boolean>(false, { nonNullable: true }),
      olderRolling: new FormControl<boolean>(false, { nonNullable: true }),
      olderDate: new FormControl<moment.Moment | null>(finishDate),
      olderDays: new FormControl<number | null>(0),
      lockNewer: new FormControl<boolean>(false, { nonNullable: true }),
      newerRolling: new FormControl<boolean>(false, { nonNullable: true }),
      newerDate: new FormControl<moment.Moment | null>(finishDate),
      newerDays: new FormControl<number | null>(0),
      accountTypes: new FormArray<FormGroup<{ accountType: FormControl<boolean> }>>([]),
      voucherTypes: new FormArray<FormGroup<{ voucherType: FormControl<boolean> }>>([]),
    });
    this.lockInformation = [];
    this.rebaseDataForm = new FormGroup({
      date: new FormControl<moment.Moment>(moment(), { nonNullable: true }),
    });

    this.listenToLockForm();
    this.maintenance = { enabled: false, user: '' };
    this.version = environment.version;
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as {
        maintenance: { enabled: boolean; user: string };
        lockInformation: [LockInfo];
        accountTypes: AccountType[];
        voucherTypes: AccountType[];
      };
      this.accountTypes = data.accountTypes;
      this.voucherTypes = data.voucherTypes;
      this.showLockInformation(data.lockInformation);
      this.maintenance = data.maintenance;
    });
  }

  showLockInformation(info: LockInfo[]) {
    this.lockInformation = info;
    this.lockInfoForm.controls.accountTypes.reset();
    this.accountTypes.forEach((x) =>
      this.lockInfoForm.controls.accountTypes.push(
        new FormGroup({
          accountType: new FormControl<boolean>(false, { nonNullable: true }),
        }),
      ),
    );
    this.lockInfoForm.controls.voucherTypes.reset();
    this.voucherTypes.forEach((x) =>
      this.lockInfoForm.controls.voucherTypes.push(
        new FormGroup({
          voucherType: new FormControl<boolean>(false, { nonNullable: true }),
        }),
      ),
    );
    this.dataSource = new LockDataSource(this.lockObservable);
    this.lockObservable.next(this.lockInformation);
  }

  listenToLockForm() {
    this.lockInfoForm.controls.lockOlder.valueChanges.subscribe((x) => {
      this.lockOlder = x;
    });
    this.lockInfoForm.controls.lockNewer.valueChanges.subscribe((x) => {
      this.lockNewer = x;
    });
    this.lockInfoForm.controls.olderRolling.valueChanges.subscribe((x) => {
      this.olderRolling = x;
    });
    this.lockInfoForm.controls.newerRolling.valueChanges.subscribe((x) => {
      this.newerRolling = x;
    });
  }

  saveLock() {
    const item = new LockInfo();
    if (this.lockOlder && this.olderRolling) {
      item.start.days = +this.lockInfoForm.value.olderDays!;
    } else if (this.lockOlder && !this.olderRolling) {
      item.start.date = this.lockInfoForm.value.olderDate!.format('DD-MMM-YYYY');
    }
    if (this.lockNewer && this.newerRolling) {
      item.finish.days = +this.lockInfoForm.value.newerDays!;
    }
    if (this.lockNewer && !this.newerRolling) {
      item.finish.date = this.lockInfoForm.value.newerDate!.format('DD-MMM-YYYY');
    }
    const atArray = this.lockInfoForm.controls.accountTypes;
    this.accountTypes.forEach((at, index) => {
      if (atArray.controls[index].value.accountType) {
        item.accountTypes.push({ id: at.id, name: at.name });
      }
    });

    const vtArray = this.lockInfoForm.controls.voucherTypes;
    this.voucherTypes.forEach((vt, index) => {
      if (vtArray.controls[index].value.voucherType) {
        item.voucherTypes.push({ id: vt.id, name: vt.name });
      }
    });

    const validFrom = this.lockInfoForm.value.validFrom;
    if (validFrom) {
      item.validFrom = moment(validFrom).format('DD-MMM-YYYY');
    }

    const validTill = this.lockInfoForm.value.validTill;
    if (validTill) {
      item.validTill = moment(validTill).format('DD-MMM-YYYY');
    }
    item.index = +this.lockInfoForm.value.index!;

    this.ser.setLockInformation(item).subscribe(
      (result) => {
        this.showLockInformation(result);
        this.toaster.show('Success', 'Lock information Updated');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  deleteLock(row: LockInfo) {
    this.ser.deleteLockInformation(row.id as string).subscribe((x) => {
      this.showLockInformation(x);
    });
  }

  confirmRebase(): void {
    const rebaseDate = this.rebaseDataForm.value.date!.format('DD-MMM-YYYY');
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: {
        title: 'Rebase the database?',
        content: `Are you sure you want to rebase the database as on ${rebaseDate}? This action is destructive and cannot be undone.`,
      },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.rebaseData(rebaseDate);
      }
    });
  }

  rebaseData(rebaseDate: string) {
    this.ser.rebaseDatabase(rebaseDate).subscribe(
      () => {
        this.toaster.show('Success', 'Data has been rebased!');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  toggleMaintenance() {
    this.ser.setMaintenance(!this.maintenance.enabled).subscribe((x) => {
      this.maintenance = x;
    });
  }
}

==> v11.0.1.tar.gz: brewman/overlord/src/app/settings/settings.component.ts <==
import { Component, OnInit } from '@angular/core';
import { FormArray, FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { BehaviorSubject } from 'rxjs';

import { environment } from '../../environments/environment';
import { AuthService } from '../auth/auth.service';
import { AccountType } from '../core/account-type';
import { ToasterService } from '../core/toaster.service';
import { ConfirmDialogComponent } from '../shared/confirm-dialog/confirm-dialog.component';

import { LockDataSource } from './lock-datasource';
import { LockInfo } from './lock-info';
import { SettingsService } from './settings.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.css'],
})
export class SettingsComponent implements OnInit {
  public lockObservable = new BehaviorSubject<LockInfo[]>([]);
  dataSource: LockDataSource = new LockDataSource(this.lockObservable);

  accountTypes: AccountType[] = [];
  voucherTypes: AccountType[] = [];

  lockInfoForm: FormGroup<{
    validFrom: FormControl<moment.Moment | null>;
    validTill: FormControl<moment.Moment | null>;
    index: FormControl<number>;
    lockOlder: FormControl<boolean>;
    olderRolling: FormControl<boolean>;
    olderDate: FormControl<moment.Moment | null>;
    olderDays: FormControl<number | null>;
    lockNewer: FormControl<boolean>;
    newerRolling: FormControl<boolean>;
    newerDate: FormControl<moment.Moment | null>;
    newerDays: FormControl<number | null>;
    accountTypes: FormArray<FormGroup<{ accountType: FormControl<boolean> }>>;
    voucherTypes: FormArray<FormGroup<{ voucherType: FormControl<boolean> }>>;
  }>;

  lockInformation: LockInfo[];
  lockOlder = false;
  olderRolling = false;
  lockNewer = false;
  newerRolling = false;

  rebaseDataForm: FormGroup<{
    date: FormControl<moment.Moment>;
  }>;

  maintenance: { enabled: boolean; user: string };

  displayedColumns = ['index', 'validity', 'voucherTypes', 'accountTypes', 'lock', 'action'];

  version: string;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    public auth: AuthService,
    private ser: SettingsService,
  ) {
    const startDate = moment().date(1);
    const finishDate = moment().date(startDate.daysInMonth());
    this.lockInfoForm = new FormGroup({
      validFrom: new FormControl<moment.Moment | null>(null),
      validTill: new FormControl<moment.Moment | null>(null),
      index: new FormControl<number>(0, { nonNullable: true }),
      lockOlder: new FormControl<boolean>(false, { nonNullable: true }),
      olderRolling: new FormControl<boolean>(false, { nonNullable: true }),
      olderDate: new FormControl<moment.Moment | null>(finishDate),
      olderDays: new FormControl<number | null>(0),
      lockNewer: new FormControl<boolean>(false, { nonNullable: true }),
      newerRolling: new FormControl<boolean>(false, { nonNullable: true }),
      newerDate: new FormControl<moment.Moment | null>(finishDate),
      newerDays: new FormControl<number | null>(0),
      accountTypes: new FormArray<FormGroup<{ accountType: FormControl<boolean> }>>([]),
      voucherTypes: new FormArray<FormGroup<{ voucherType: FormControl<boolean> }>>([]),
    });
    this.lockInformation = [];
    this.rebaseDataForm = new FormGroup({
      date: new FormControl<moment.Moment>(moment(), { nonNullable: true }),
    });

    this.listenToLockForm();
    this.maintenance = { enabled: false, user: '' };
    this.version = environment.version;
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as {
        maintenance: { enabled: boolean; user: string };
        lockInformation: [LockInfo];
        accountTypes: AccountType[];
        voucherTypes: AccountType[];
      };
      this.accountTypes = data.accountTypes;
      this.voucherTypes = data.voucherTypes;
      this.showLockInformation(data.lockInformation);
      this.maintenance = data.maintenance;
    });
  }

  showLockInformation(info: LockInfo[]) {
    this.lockInformation = info;
    this.lockInfoForm.controls.accountTypes.clear();
    this.accountTypes.forEach((x) =>
      this.lockInfoForm.controls.accountTypes.push(
        new FormGroup({
          accountType: new FormControl<boolean>(false, { nonNullable: true }),
        }),
      ),
    );
    this.lockInfoForm.controls.voucherTypes.clear();
    this.voucherTypes.forEach((x) =>
      this.lockInfoForm.controls.voucherTypes.push(
        new FormGroup({
          voucherType: new FormControl<boolean>(false, { nonNullable: true }),
        }),
      ),
    );
    this.dataSource = new LockDataSource(this.lockObservable);
    this.lockObservable.next(this.lockInformation);
  }

  listenToLockForm() {
    this.lockInfoForm.controls.lockOlder.valueChanges.subscribe((x) => {
      this.lockOlder = x;
    });
    this.lockInfoForm.controls.lockNewer.valueChanges.subscribe((x) => {
      this.lockNewer = x;
    });
    this.lockInfoForm.controls.olderRolling.valueChanges.subscribe((x) => {
      this.olderRolling = x;
    });
    this.lockInfoForm.controls.newerRolling.valueChanges.subscribe((x) => {
      this.newerRolling = x;
    });
  }

  saveLock() {
    const item = new LockInfo();
    if (this.lockOlder && this.olderRolling) {
      item.start.days = this.lockInfoForm.value.olderDays ?? 0;
    } else if (this.lockOlder && !this.olderRolling) {
      item.start.date = (this.lockInfoForm.value.olderDate ?? moment()).format('DD-MMM-YYYY');
    }
    if (this.lockNewer && this.newerRolling) {
      item.finish.days = this.lockInfoForm.value.newerDays ?? 0;
    }
    if (this.lockNewer && !this.newerRolling) {
      item.finish.date = (this.lockInfoForm.value.newerDate ?? moment()).format('DD-MMM-YYYY');
    }
    const atArray = this.lockInfoForm.controls.accountTypes;
    this.accountTypes.forEach((at, index) => {
      if (atArray.controls[index].value.accountType) {
        item.accountTypes.push({ id: at.id, name: at.name });
      }
    });

    const vtArray = this.lockInfoForm.controls.voucherTypes;
    this.voucherTypes.forEach((vt, index) => {
      if (vtArray.controls[index].value.voucherType) {
        item.voucherTypes.push({ id: vt.id, name: vt.name });
      }
    });

    const validFrom = this.lockInfoForm.value.validFrom;
    if (validFrom) {
      item.validFrom = moment(validFrom).format('DD-MMM-YYYY');
    }

    const validTill = this.lockInfoForm.value.validTill;
    if (validTill) {
      item.validTill = moment(validTill).format('DD-MMM-YYYY');
    }
    item.index = this.lockInfoForm.value.index ?? 0;

    this.ser.setLockInformation(item).subscribe(
      (result) => {
        this.showLockInformation(result);
        this.toaster.show('Success', 'Lock information Updated');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  deleteLock(row: LockInfo) {
    this.ser.deleteLockInformation(row.id as string).subscribe((x) => {
      this.showLockInformation(x);
    });
  }

  confirmRebase(): void {
    const rebaseDate = (this.rebaseDataForm.value.date ?? moment()).format('DD-MMM-YYYY');
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: {
        title: 'Rebase the database?',
        content: `Are you sure you want to rebase the database as on ${rebaseDate}? This action is destructive and cannot be undone.`,
      },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.rebaseData(rebaseDate);
      }
    });
  }

  rebaseData(rebaseDate: string) {
    this.ser.rebaseDatabase(rebaseDate).subscribe(
      () => {
        this.toaster.show('Success', 'Data has been rebased!');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  toggleMaintenance() {
    this.ser.setMaintenance(!this.maintenance.enabled).subscribe((x) => {
      this.maintenance = x;
    });
  }
}
